import 'dart:typed_data';

import 'package:flutter/services.dart';
import 'package:sunmi_printer_v2/src/constants.dart';

/// Interface between flutter and Sunmi printer
class SunmiPrinterV2 {
  static const MethodChannel _channel = const MethodChannel('sunmi_printer_v2');

  /// Returns the serialNumber of the device if available.
  static Future<String?> get getSerialNo async {
    final String? serialNo = await _channel.invokeMethod('getSerialNo');
    return serialNo;
  }

  /// Returns the printerSerialNumber of the device if available.
  static Future<String?> get getPrinterSerialNo async {
    final String? printerSerialNo =
        await _channel.invokeMethod('getPrinterSerialNo');
    return printerSerialNo;
  }

  /// Returns the printer model of the device if available.
  static Future<String?> get getPrinterModel async {
    final String? model = await _channel.invokeMethod('getPrinterModel');
    return model;
  }

  /// Returns the printerVersion of the device if available.
  static Future<String?> get getPrinterVersion async {
    final String? printerVersion =
        await _channel.invokeMethod('getPrinterVersion');
    return printerVersion;
  }

  /// Returns the printer paper type of the device if available.
  static Future<String?> get getPrinterPaper async {
    final String? paper = await _channel.invokeMethod('getPrinterPaper');
    return paper;
  }

  /// Initialize the printer. This will clear all set alignments and font changes to default.
  static Future<void> initPrinter() async {
    await _channel.invokeMethod("initPrinter");
  }

  /// This will feed the paper.
  static Future<void> feedPaper() async {
    await _channel.invokeMethod("feedPaper");
  }

  /// Print the text.
  /// [content] is mandatory
  /// [size] Font size of the text to printed.
  /// [isBold] Boolean to set whether text needs to printed in bold.
  /// [isUnderline] Boolean to set whether text needs to be underlined.
  static Future<void> printText(String content,
      {int size = 18, bool isBold = false, bool isUnderline: false}) async {
    await _channel.invokeMethod("printText", <String, dynamic>{
      'content': content,
      'size': size,
      'isBold': isBold,
      'isUnderline': isUnderline,
    });
  }

  /// Set the TextAlign [alignment] for future prints.
  /// Eg: [PrinterAlignment.center]
  static Future<void> setAlign(PrinterTextAlignment alignment) async {
    await _channel.invokeMethod("setAlign", <String, dynamic>{
      'alignment': alignment.value,
    });
  }

  /// Set the font bold for future prints.
  /// [isBold] boolean.
  static Future<void> setBold(bool isBold) async {
    await _channel.invokeMethod("setBold", <String, dynamic>{
      'isBold': isBold,
    });
  }

  static Future<void> setAntiWhite(bool isAntiWhite) async {
    await _channel.invokeMethod("setAntiWhite", <String, dynamic>{
      'isAntiWhite': isAntiWhite,
    });
  }

  /// Prints a barcode
  /// [barcode],  [barcodeType]  must be provided.
  /// [height] and [width] can be customized.
  /// Defaults are [height] = 30 and [width] = 3
  static Future<void> printBarcode(
      {required String barcode,
      BarcodeType? type,
      int height = 30,
      int width = 3,
      BarcodeTextPosition textPosition = BarcodeTextPosition.noText}) async {
    await _channel.invokeMethod("printBarcode", <String, dynamic>{
      'barcode': barcode,
      "type": type?.value,
      'textPosition': textPosition.value,
      'width': width,
      'height': height,
    });
  }

  /// Print texts as a table.
  /// [texts] can be sent as a array.
  /// Column width ratio can be set as [widths] array. The default is [1,1]
  /// Text alignment can be set as [alignment] array.
  static Future<void> printTable({
    List<String>? texts,
    List<int> widths = const [1, 1],
    List<int> alignments = const [0, 2],
  }) async {
    await _channel.invokeMethod('printTable', <String, dynamic>{
      'texts': texts,
      "widths": Int32List.fromList(widths),
      "alignments": Int32List.fromList(alignments),
    });
  }

  static Future<void> printLineDivider() async {
    await _channel.invokeMethod("printLineDivider");
  }

  /// Set the line warping.
  /// Move the printing to a nex line
  /// Default it moves to 1 line but you can set lines you want[lines]
  static Future<void> lineWrap({int lines = 1}) async {
    await _channel.invokeMethod("lineWrap", <String, dynamic>{
      "lines": lines,
    });
  }

  /// Print a sample printout
  @deprecated
  static Future<void> printSample() async {
    await _channel.invokeMethod("printSample");
  }
}
